﻿using System.Collections.Generic;
using System.IO;
using System.Net.Http;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;

using Newtonsoft.Json;

namespace TestProject;

/// <summary>
/// 챗봇 클라이언트
/// </summary>
public class ChatBotClient
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 기준 URL
    /// </summary>
    private readonly string baseURL;

    /// <summary>
    /// HTTP 클라이언트
    /// </summary>
    private readonly HttpClient httpClient;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - ChatBotClient(baseURL)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="baseURL">기준 URL</param>
    public ChatBotClient(string baseURL)
    {
        this.baseURL    = baseURL;
        this.httpClient = new HttpClient();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 스트리밍 메시지 비동기 열거 가능형 요청하기 (비동기) - RequestStreamingMessageAsyncEnumerableAsync(requestMessage, cancellationToken)

    /// <summary>
    /// 스트리밍 메시지 비동기 열거 가능형 요청하기 (비동기)
    /// </summary>
    /// <param name="requestMessage">요청 메시지</param>
    /// <param name="cancellationToken">취소 토큰</param>
    /// <returns>스트리밍 메시지 비동기 열거 가능형</returns>
    public async IAsyncEnumerable<AIMessageChunk> RequestStreamingMessageAsyncEnumerableAsync(RequestMessage requestMessage, [EnumeratorCancellation] CancellationToken cancellationToken = default)
    {
        StringContent stringContent = new(JsonConvert.SerializeObject(requestMessage), Encoding.UTF8, "application/json");

        using HttpRequestMessage httpRequestMessage = new(HttpMethod.Post, baseURL) { Content = stringContent };

        using HttpResponseMessage httpResponseMessage = await httpClient.SendAsync(httpRequestMessage, HttpCompletionOption.ResponseHeadersRead, cancellationToken);

        if(httpResponseMessage.IsSuccessStatusCode)
        {
            using Stream stream = await httpResponseMessage.Content.ReadAsStreamAsync(cancellationToken);

            using StreamReader streamReader = new(stream);

            string line;

            while((line = await streamReader.ReadLineAsync(cancellationToken)) != null)
            {
                if(cancellationToken.IsCancellationRequested)
                {
                    yield break;
                }

                if(line == "data: [DONE]")
                {
                    break;
                }

                if(line.StartsWith("data: "))
                {
                    AIMessageChunk aiMessageChunk = JsonConvert.DeserializeObject<AIMessageChunk>(line[6..]);

                    yield return aiMessageChunk;
                }
            }
        }
    }

    #endregion
}